@@ -1,9 +1,10 @@
-#   Copyright 2013 Oli Schacher
+# -*- coding: utf-8 -*-
+#   Copyright 2009-2021 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# 
+#
 # http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
@@ -13,6 +14,9 @@
 # limitations under the License.
 #
 #
-__all__=['bounce','core','daemon','funkyconsole','protocolbase','scansession','shared','stats','threadpool']
+from fuglu.utils.version import get_version, VERSION_ALPHA, VERSION_BETA, VERSION_RC, VERSION_FINAL
 
-FUGLU_VERSION="0.6.2"
+
+FUGLU_VERSION = (0, 10, 8, VERSION_FINAL, 0)
+
+__version__ = get_version()
