@@ -1,3 +1,4 @@
+# -*- coding: UTF-8 -*-
 # $Id: ppymilterbase.py 33 2009-04-08 20:40:02Z codewhale $
 # ==============================================================================
 # Copyright 2008 Google Inc.
@@ -27,89 +28,86 @@ __author__ = 'Eric DeFriez'
 
 import binascii
 import logging
-import os
-import socket
 import struct
-import sys
-import types
+from fuglu.stringencode import force_bString, force_uString
 
 
-MILTER_VERSION = 2 # Milter version we claim to speak (from pmilter)
+MILTER_VERSION = 2  # Milter version we claim to speak (from pmilter)
 
 # Potential milter command codes and their corresponding PpyMilter callbacks.
 # From sendmail's include/libmilter/mfdef.h
-SMFIC_ABORT   = 'A' # "Abort"
-SMFIC_BODY    = 'B' # "Body chunk"
-SMFIC_CONNECT = 'C' # "Connection information"
-SMFIC_MACRO   = 'D' # "Define macro"
-SMFIC_BODYEOB = 'E' # "final body chunk (End)"
-SMFIC_HELO    = 'H' # "HELO/EHLO"
-SMFIC_HEADER  = 'L' # "Header"
-SMFIC_MAIL    = 'M' # "MAIL from"
-SMFIC_EOH     = 'N' # "EOH"
-SMFIC_OPTNEG  = 'O' # "Option negotation"
-SMFIC_RCPT    = 'R' # "RCPT to"
-SMFIC_QUIT    = 'Q' # "QUIT"
-SMFIC_DATA    = 'T' # "DATA"
-SMFIC_UNKNOWN = 'U' # "Any unknown command"
+SMFIC_ABORT = b'A'  # "Abort"
+SMFIC_BODY = b'B'  # "Body chunk"
+SMFIC_CONNECT = b'C'  # "Connection information"
+SMFIC_MACRO = b'D'  # "Define macro"
+SMFIC_BODYEOB = b'E'  # "final body chunk (End)"
+SMFIC_HELO = b'H'  # "HELO/EHLO"
+SMFIC_HEADER = b'L'  # "Header"
+SMFIC_MAIL = b'M'  # "MAIL from"
+SMFIC_EOH = b'N'  # "EOH"
+SMFIC_OPTNEG = b'O'  # "Option negotation"
+SMFIC_RCPT = b'R'  # "RCPT to"
+SMFIC_QUIT = b'Q'  # "QUIT"
+SMFIC_DATA = b'T'  # "DATA"
+SMFIC_UNKNOWN = b'U'  # "Any unknown command"
 
 COMMANDS = {
-  SMFIC_ABORT: 'Abort',
-  SMFIC_BODY: 'Body',
-  SMFIC_CONNECT: 'Connect',
-  SMFIC_MACRO: 'Macro',
-  SMFIC_BODYEOB: 'EndBody',
-  SMFIC_HELO: 'Helo',
-  SMFIC_HEADER: 'Header',
-  SMFIC_MAIL: 'MailFrom',
-  SMFIC_EOH: 'EndHeaders',
-  SMFIC_OPTNEG: 'OptNeg',
-  SMFIC_RCPT: 'RcptTo',
-  SMFIC_QUIT: 'Quit',
-  SMFIC_DATA: 'Data',
-  SMFIC_UNKNOWN: 'Unknown',
-  }
+    SMFIC_ABORT: b'Abort',
+    SMFIC_BODY: b'Body',
+    SMFIC_CONNECT: b'Connect',
+    SMFIC_MACRO: b'Macro',
+    SMFIC_BODYEOB: b'EndBody',
+    SMFIC_HELO: b'Helo',
+    SMFIC_HEADER: b'Header',
+    SMFIC_MAIL: b'MailFrom',
+    SMFIC_EOH: b'EndHeaders',
+    SMFIC_OPTNEG: b'OptNeg',
+    SMFIC_RCPT: b'RcptTo',
+    SMFIC_QUIT: b'Quit',
+    SMFIC_DATA: b'Data',
+    SMFIC_UNKNOWN: b'Unknown',
+}
 
 # To register/mask callbacks during milter protocol negotiation with sendmail.
 # From sendmail's include/libmilter/mfdef.h
 NO_CALLBACKS = 127  # (all seven callback flags set: 1111111)
 CALLBACKS = {
-  'OnConnect':    1,  # 0x01 SMFIP_NOCONNECT # Skip SMFIC_CONNECT
-  'OnHelo':       2,  # 0x02 SMFIP_NOHELO    # Skip SMFIC_HELO
-  'OnMailFrom':   4,  # 0x04 SMFIP_NOMAIL    # Skip SMFIC_MAIL
-  'OnRcptTo':     8,  # 0x08 SMFIP_NORCPT    # Skip SMFIC_RCPT
-  'OnBody':       16, # 0x10 SMFIP_NOBODY    # Skip SMFIC_BODY
-  'OnHeader':     32, # 0x20 SMFIP_NOHDRS    # Skip SMFIC_HEADER
-  'OnEndHeaders': 64, # 0x40 SMFIP_NOEOH     # Skip SMFIC_EOH
-  }
+    'OnConnect':    1,  # 0x01 SMFIP_NOCONNECT # Skip SMFIC_CONNECT
+    'OnHelo':       2,  # 0x02 SMFIP_NOHELO    # Skip SMFIC_HELO
+    'OnMailFrom':   4,  # 0x04 SMFIP_NOMAIL    # Skip SMFIC_MAIL
+    'OnRcptTo':     8,  # 0x08 SMFIP_NORCPT    # Skip SMFIC_RCPT
+    'OnBody':       16,  # 0x10 SMFIP_NOBODY    # Skip SMFIC_BODY
+    'OnHeader':     32,  # 0x20 SMFIP_NOHDRS    # Skip SMFIC_HEADER
+    'OnEndHeaders': 64,  # 0x40 SMFIP_NOEOH     # Skip SMFIC_EOH
+}
 
 # Acceptable response commands/codes to return to sendmail (with accompanying
 # command data).  From sendmail's include/libmilter/mfdef.h
 RESPONSE = {
-    'ADDRCPT'    : '+', # SMFIR_ADDRCPT    # "add recipient"
-    'DELRCPT'    : '-', # SMFIR_DELRCPT    # "remove recipient"
-    'ACCEPT'     : 'a', # SMFIR_ACCEPT     # "accept"
-    'REPLBODY'   : 'b', # SMFIR_REPLBODY   # "replace body (chunk)"
-    'CONTINUE'   : 'c', # SMFIR_CONTINUE   # "continue"
-    'DISCARD'    : 'd', # SMFIR_DISCARD    # "discard"
-    'CONNFAIL'   : 'f', # SMFIR_CONN_FAIL  # "cause a connection failure"
-    'ADDHEADER'  : 'h', # SMFIR_ADDHEADER  # "add header"
-    'INSHEADER'  : 'i', # SMFIR_INSHEADER  # "insert header"
-    'CHGHEADER'  : 'm', # SMFIR_CHGHEADER  # "change header"
-    'PROGRESS'   : 'p', # SMFIR_PROGRESS   # "progress"
-    'QUARANTINE' : 'q', # SMFIR_QUARANTINE # "quarantine"
-    'REJECT'     : 'r', # SMFIR_REJECT     # "reject"
-    'SETSENDER'  : 's', # v3 only?
-    'TEMPFAIL'   : 't', # SMFIR_TEMPFAIL   # "tempfail"
-    'REPLYCODE'  : 'y', # SMFIR_REPLYCODE  # "reply code etc"
-    }
+    'ADDRCPT': '+',  # SMFIR_ADDRCPT    # "add recipient"
+    'DELRCPT': '-',  # SMFIR_DELRCPT    # "remove recipient"
+    'ACCEPT': 'a',  # SMFIR_ACCEPT     # "accept"
+    'REPLBODY': 'b',  # SMFIR_REPLBODY   # "replace body (chunk)"
+    'CONTINUE': 'c',  # SMFIR_CONTINUE   # "continue"
+    'DISCARD': 'd',  # SMFIR_DISCARD    # "discard"
+    'CONNFAIL': 'f',  # SMFIR_CONN_FAIL  # "cause a connection failure"
+    'ADDHEADER': 'h',  # SMFIR_ADDHEADER  # "add header"
+    'INSHEADER': 'i',  # SMFIR_INSHEADER  # "insert header"
+    'CHGHEADER': 'm',  # SMFIR_CHGHEADER  # "change header"
+    'PROGRESS': 'p',  # SMFIR_PROGRESS   # "progress"
+    'QUARANTINE': 'q',  # SMFIR_QUARANTINE # "quarantine"
+    'REJECT': 'r',  # SMFIR_REJECT     # "reject"
+    'SETSENDER': 's',  # v3 only?
+    'TEMPFAIL': 't',  # SMFIR_TEMPFAIL   # "tempfail"
+    'REPLYCODE': 'y',  # SMFIR_REPLYCODE  # "reply code etc"
+}
 
 
 def printchar(char):
     """Useful debugging function for milter developers."""
-    print ('char: %s [qp=%s][hex=%s][base64=%s]' %
-           (char, binascii.b2a_qp(char), binascii.b2a_hex(char),
-            binascii.b2a_base64(char)))
+    print('char: %s [qp=%s][hex=%s][base64=%s]' %
+          (char, binascii.b2a_qp(char), binascii.b2a_hex(char),
+           binascii.b2a_base64(char)))
 
 
 def CanonicalizeAddress(addr):
@@ -122,31 +120,39 @@ def CanonicalizeAddress(addr):
       The addr with leading and trailing angle brackets removed unless
       the address is "<>" (in which case the string is returned unchanged).
     """
-    if addr == '<>': return addr
+    if addr == '<>':
+        return addr
     return addr.lstrip('<').rstrip('>')
 
 
 class PpyMilterException(Exception):
+
     """Parent of all other PpyMilter exceptions.  Subclass this: do not
     construct or catch explicitly!"""
 
 
 class PpyMilterPermFailure(PpyMilterException):
+
     """Milter exception that indicates a perment failure."""
 
 
 class PpyMilterTempFailure(PpyMilterException):
+
     """Milter exception that indicates a temporary/transient failure."""
 
 
 class PpyMilterCloseConnection(PpyMilterException):
+
     """Exception that indicates the server should close the milter connection."""
 
+
 class PpyMilterActionError(PpyMilterException):
+
     """Exception raised when an action is performed that was not negotiated."""
 
 
 class PpyMilterDispatcher(object):
+
     """Dispatcher class for a milter server.  This class accepts entire
     milter commands as a string (command character + binary data), parses
     the command and binary data appropriately and invokes the appropriate
@@ -159,12 +165,11 @@ class PpyMilterDispatcher(object):
         milter_class instance.
 
         Args:
-          milter_class: A class (not an instance) that handles callbacks for
+          milter: A class (not an instance) that handles callbacks for
                         milter commands (e.g. a child of the PpyMilter class).
         """
         self.__milter = milter
 
-
     def Dispatch(self, data):
         """Callback function for the milter socket server to handle a single
         milter command.  Parses the milter command data, invokes the milter
@@ -184,34 +189,35 @@ class PpyMilterDispatcher(object):
           PpyMilterCloseConnection: Indicating the (milter) connection should
                                     be closed.
         """
-        (cmd, data) = (data[0], data[1:])
+        cmd, data = data[0], data[1:]
         try:
             if cmd not in COMMANDS:
-                logging.warn('Unknown command code: "%s" ("%s")', cmd, data)
+                logging.warn('Unknown command code: "%s" ("%s")', force_uString(cmd), force_uString(data))
                 return RESPONSE['CONTINUE']
-            command = COMMANDS[cmd]
-            parser_callback_name = '_Parse%s' % command
-            handler_callback_name = 'On%s' % command
+            u_command = force_uString(COMMANDS[cmd])  # (unicode)
+            parser_callback_name = '_Parse%s' % u_command
+            handler_callback_name = 'On%s' % u_command
 
             if not hasattr(self, parser_callback_name):
-                logging.error('No parser implemented for "%s"', command)
+                logging.error('No parser implemented for "%s"', u_command)
                 return RESPONSE['CONTINUE']
 
             if not hasattr(self.__milter, handler_callback_name):
-                logging.warn('Unimplemented command in milter %s: "%s" ("%s")'%(self.__milter, command, data))
+                logging.warn('Unimplemented command in milter %s: "%s" ("%s")' % (
+                    self.__milter, u_command, data))
                 return RESPONSE['CONTINUE']
 
             parser = getattr(self, parser_callback_name)
             callback = getattr(self.__milter, handler_callback_name)
             args = parser(cmd, data)
             return callback(*args)
-        except PpyMilterTempFailure, e:
+        except PpyMilterTempFailure as e:
             logging.info('Temp Failure: %s', str(e))
             return RESPONSE['TEMPFAIL']
-        except PpyMilterPermFailure, e:
+        except PpyMilterPermFailure as e:
             logging.info('Perm Failure: %s', str(e))
             return RESPONSE['REJECT']
-        return RESPONSE['CONTINUE']
+        #return RESPONSE['CONTINUE'] # this line is never reached
 
     def _ParseOptNeg(self, cmd, data):
         """Parse the 'OptNeg' milter data into arguments for the milter handler.
@@ -229,8 +235,8 @@ class PpyMilterDispatcher(object):
             protocol: Bitmask of the callback functions we are registering.
 
         """
-        (ver, actions, protocol) = struct.unpack('!III', data)
-        return (cmd, ver, actions, protocol)
+        ver, actions, protocol = struct.unpack('!III', data)
+        return cmd, ver, actions, protocol
 
     def _ParseMacro(self, cmd, data):
         """Parse the 'Macro' milter data into arguments for the milter handler.
@@ -245,8 +251,8 @@ class PpyMilterDispatcher(object):
             macro: The single character command code this macro is for.
             data: A list of strings alternating between name, value of macro.
         """
-        (macro, data) = (data[0], data[1:])
-        return (cmd, macro, data.split('\0'))
+        macro, data = (data[0], data[1:])
+        return cmd, macro, data.split('\0')
 
     def _ParseConnect(self, cmd, data):
         """Parse the 'Connect' milter data into arguments for the milter handler.
@@ -263,17 +269,15 @@ class PpyMilterDispatcher(object):
             port: The network port if appropriate for the connection.
             address: Remote address of the connection (e.g. IP address).
         """
-        (hostname, data) = data.split('\0', 1)
+        hostname, data = data.split('\0', 1)
         family = struct.unpack('c', data[0])[0]
-        
-        #http://code.google.com/p/ppymilter/issues/detail?id=6
-        if family==4 or family==6:
+        if family in ('4', '6'):  # SMFIA_INET / SMFIA_INET6
             port = struct.unpack('!H', data[1:3])[0]
-            address = data[3:]
-        else:
-            port =0
-            address = 'unknown'
-        return (cmd, hostname, family, port, address)
+            address, _ = data[3:].split('\0', 1)
+        else:  # SMFIA_UNKNOWN / SMFIA_UNIX
+            port = None
+            address = None
+        return cmd, hostname, family, port, address
 
     def _ParseHelo(self, cmd, data):
         """Parse the 'Helo' milter data into arguments for the milter handler.
@@ -287,7 +291,9 @@ class PpyMilterDispatcher(object):
             cmd: The single character command code representing this command.
             data: TODO: parse this better
         """
-        return (cmd, data)
+        data = data.split('\0')[0]
+
+        return cmd, data
 
     def _ParseMailFrom(self, cmd, data):
         """Parse the 'MailFrom' milter data into arguments for the milter handler.
@@ -302,8 +308,8 @@ class PpyMilterDispatcher(object):
             mailfrom: The canonicalized MAIL From email address.
             esmtp_info: Extended SMTP (esmtp) info as a list of strings.
         """
-        (mailfrom, esmtp_info) = data.split('\0', 1)
-        return (cmd, CanonicalizeAddress(mailfrom), esmtp_info.split('\0'))
+        mailfrom, esmtp_info = data.split('\0', 1)
+        return cmd, CanonicalizeAddress(mailfrom), esmtp_info.split('\0')
 
     def _ParseRcptTo(self, cmd, data):
         """Parse the 'RcptTo' milter data into arguments for the milter handler.
@@ -318,8 +324,8 @@ class PpyMilterDispatcher(object):
             rcptto: The canonicalized RCPT To email address.
             esmtp_info: Extended SMTP (esmtp) info as a list of strings.
         """
-        (rcptto, esmtp_info) = data.split('\0', 1)
-        return (cmd, CanonicalizeAddress(rcptto), esmtp_info.split('\0'))
+        rcptto, esmtp_info = data.split('\0', 1)
+        return cmd, CanonicalizeAddress(rcptto), esmtp_info.split('\0')
 
     def _ParseHeader(self, cmd, data):
         """Parse the 'Header' milter data into arguments for the milter handler.
@@ -334,8 +340,8 @@ class PpyMilterDispatcher(object):
             key: The name of the header.
             val: The value/data for the header.
         """
-        (key, val) = data.split('\0', 1)
-        return (cmd, key, val)
+        key, val = data.split('\0', 1)
+        return cmd, key, val
 
     def _ParseEndHeaders(self, cmd, data):
         """Parse the 'EndHeaders' milter data into arguments for the milter handler.
@@ -348,7 +354,7 @@ class PpyMilterDispatcher(object):
           A tuple (cmd) where:
             cmd: The single character command code representing this command.
         """
-        return (cmd)
+        return cmd
 
     def _ParseBody(self, cmd, data):
         """Parse the 'Body' milter data into arguments for the milter handler.
@@ -362,7 +368,7 @@ class PpyMilterDispatcher(object):
             cmd : The single character command code representing this command.
             data: TODO: parse this better
         """
-        return (cmd, data)
+        return cmd, data
 
     def _ParseEndBody(self, cmd, data):
         """Parse the 'EndBody' milter data into arguments for the milter handler.
@@ -375,7 +381,7 @@ class PpyMilterDispatcher(object):
           A tuple (cmd) where:
             cmd: The single character command code representing this command.
         """
-        return (cmd)
+        return cmd
 
     def _ParseQuit(self, cmd, data):
         """Parse the 'Quit' milter data into arguments for the milter handler.
@@ -388,7 +394,7 @@ class PpyMilterDispatcher(object):
           A tuple (cmd) where:
             cmd: The single character command code representing this command.
         """
-        return (cmd)
+        return cmd
 
     def _ParseAbort(self, cmd, data):
         """Parse the 'Abort' milter data into arguments for the milter handler.
@@ -401,13 +407,15 @@ class PpyMilterDispatcher(object):
           A tuple (cmd) where:
             cmd: The single character command code representing this command.
         """
-        return (cmd)
+        return cmd
+
+    def _ParseData(self, cmd, data):
+        # print "pdata: cmd=%s data=%s"%(cmd,data)
+        return cmd, data
 
-    def _ParseData(self,cmd,data):
-        #print "pdata: cmd=%s data=%s"%(cmd,data)
-        return (cmd,data)
 
 class PpyMilter(object):
+
     """Pure python milter handler base class.  Inherit from this class
     and override any On*() commands you would like your milter to handle.
     Register any actions your milter may perform using the Can*() functions
@@ -422,12 +430,12 @@ class PpyMilter(object):
     # Actions we tell sendmail we may perform
     # PpyMilter users invoke self.CanFoo() during their __init__()
     # to toggle these settings.
-    ACTION_ADDHDRS    = 1  # 0x01 SMFIF_ADDHDRS    # Add headers
-    ACTION_CHGBODY    = 2  # 0x02 SMFIF_CHGBODY    # Change body chunks
-    ACTION_ADDRCPT    = 4  # 0x04 SMFIF_ADDRCPT    # Add recipients
-    ACTION_DELRCPT    = 8  # 0x08 SMFIF_DELRCPT    # Remove recipients
-    ACTION_CHGHDRS    = 16 # 0x10 SMFIF_CHGHDRS    # Change or delete headers
-    ACTION_QUARANTINE = 32 # 0x20 SMFIF_QUARANTINE # Quarantine message
+    ACTION_ADDHDRS = 1  # 0x01 SMFIF_ADDHDRS    # Add headers
+    ACTION_CHGBODY = 2  # 0x02 SMFIF_CHGBODY    # Change body chunks
+    ACTION_ADDRCPT = 4  # 0x04 SMFIF_ADDRCPT    # Add recipients
+    ACTION_DELRCPT = 8  # 0x08 SMFIF_DELRCPT    # Remove recipients
+    ACTION_CHGHDRS = 16  # 0x10 SMFIF_CHGHDRS    # Change or delete headers
+    ACTION_QUARANTINE = 32  # 0x20 SMFIF_QUARANTINE # Quarantine message
 
     def __init__(self):
         """Construct a PpyMilter object.  Sets callbacks and registers
@@ -437,7 +445,7 @@ class PpyMilter(object):
         """
         self.__actions = 0
         self.__protocol = NO_CALLBACKS
-        for (callback, flag) in CALLBACKS.iteritems():
+        for (callback, flag) in CALLBACKS.items():
             if hasattr(self, callback):
                 self.__protocol &= ~flag
 
@@ -590,13 +598,13 @@ class PpyMilter(object):
         out = struct.pack('!III', MILTER_VERSION,
                           self.__actions & actions,
                           self.__protocol & protocol)
-        return cmd+out
+        return cmd + out
 
     def OnMacro(self, cmd, macro_cmd, data):
         """Callback for the 'Macro' milter command: no response required."""
         return None
 
-    def OnData(self,cmd,data):
+    def OnData(self, cmd, data):
         return self.Continue()
 
     def OnQuit(self, cmd):
@@ -672,5 +680,5 @@ class PpyMilter(object):
     def __VerifyCapability(self, action):
         if not (self.__actions & action):
             logging.error('Error: Attempted to perform an action that was not' +
-                           'requested.')
+                          'requested.')
             raise PpyMilterActionError('Action not requested in __init__')
