@@ -1,9 +1,10 @@
-#   Copyright 2014 Oli Schacher
+# -*- coding: UTF-8 -*-
+#   Copyright 2009-2021 Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# 
+#
 # http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
@@ -12,4 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-__all__ = ['archive','attachment','actionoverride','clamav','sa','p_debug','p_skipper','p_fraction','vacation','icap','sssp','fprot','domainauth','script']
\ No newline at end of file
+__all__ = ['archive', 'attachment', 'actionoverride', 'clamav', 'sa', 'p_debug',
+           'p_skipper', 'p_fraction', 'vacation', 'icap', 'sssp', 'fprot', 'drweb', 'domainauth', 'script', 'a_statsd',
+           'fuzor', 'rspamd', 'killer', 'imapcopy', 'uriextract']
